// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

part of omnitest.data;

/// These keys cannot be used as omnibusId for messages that are tested
/// with this program (seriously, why would you ever use these?).
const reservedKeys = const ['expect', 'module-responses', 'mocked-responses'];

/// Get the first key in $map that can be identified as omnibusId given that
/// $ids contains all omnibusId's that can be used.
String getOmnibusId(Iterable<String> allIds, YamlMap map) {
  var ids = map.keys.where((String key) => allIds.contains(key));
  return ids.length == 1 ? ids.first : null;
}

/// Message data post-processing
///
/// In this function omnitest.utils directives are applied. The available
/// directives are:
/// - `omnitest.utils.stringFromFile($filepath)`
Map postProcessMap(Map data) {
  return new Map.fromIterable(data.keys,
      key: (dynamic key) => key,
      value: (dynamic key) => postProcessObject(data[key]));
}

/// Same as [postProcessMap] but for lists.
List postProcessList(List data) {
  return new List.generate(data.length, (int i) {
    return postProcessObject(data[i]);
  });
}

/// Directive to load the contents of a string from a file.
const stringFromFileDirective = 'omnitest.utils.stringFromFile(';

/// Internal function for [postProcessMap] and [postProcessList].
dynamic postProcessObject(dynamic object) {
  if (object is String) {
    // Process directive.
    if (object.startsWith(stringFromFileDirective)) {
      // Get file path.
      var path =
          object.substring(stringFromFileDirective.length, object.length - 1);

      // Read file.
      var str = new File(path).readAsStringSync();

      // Return this string.
      return str;
    } else {
      // Just return the string.
      return object;
    }
  } else if (object is List) {
    // Process list recursively.
    return postProcessList(object);
  } else if (object is Map) {
    // Process map recursively.
    return postProcessMap(object);
  } else {
    // Other types are just copied
    return object;
  }
}

/// Get message data from the given YamlNode using the given omnibusId. Message
/// data can be directly located under the omnibusId key, but it can also be
/// located in a separate file that is linked using a path string.
Map<String, dynamic> resolveMessageData(YamlMap map, String omnibusId) {
  // First try to parse as map.
  if (map[omnibusId] is YamlMap) {
    return pdsFromMap(postProcessMap(map[omnibusId]));
  } else if (map[omnibusId] is String) {
    // Try to read this file path.
    try {
      var fileContent = new File(map[omnibusId]).readAsStringSync();
      var yamlContent = loadYaml(fileContent);
      if (yamlContent is YamlMap) {
        return pdsFromMap(postProcessMap(yamlContent));
      } else {
        return new Map<String, dynamic>();
      }
    } catch (e) {
      return new Map<String, dynamic>();
    }
  } else {
    return new Map<String, dynamic>();
  }
}
